package com.nosum.deliver.author.service;

import com.nosum.common.base.domain.PageInfo;
import com.nosum.deliver.author.domain.po.Author;
import org.bson.types.ObjectId;

/**
 * 作者信息服务层.
 *
 * @author Young
 */
public interface AuthorService {

    /**
     * 保存作者信息.
     *
     * @param author 作者信息
     */
    void save(Author author);


    /**
     * 分页查询作者信息.
     *
     * @param pageInfo 分页查询条件
     * @param author   作者信息
     * @return 作者信息列表
     */
    PageInfo<Author> getPage(PageInfo<Author> pageInfo, Author author);

    /**
     * 根据ID删除作者信息
     *
     * @param id id
     */
    void remove(ObjectId id);


    /**
     * 统计当天创建的作者数量.
     *
     * @return 作者数量
     */
    Long countCreatedByToDay();

    /**
     * 统计全部的作者数量.
     *
     * @return 作者数量
     */
    Long count();

    /**
     * 根据 ID 查询作者信息.
     *
     * @param id ID
     * @return 作者信息
     */
    Author getById(ObjectId id);
}
